@@ -146,16 +146,16 @@ class Deep_Supervision_UNet3d_Narrow(nn.Module):
 
 
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 3, 1, 1)
+            nn.Conv3d(32, final_output_channels, 1, 1, 0) #DeepLab Architecture로 수정
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, final_output_channels, 3, 1, 1)
+            nn.Conv3d(64, final_output_channels, 1, 1, 0)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, final_output_channels, 3, 1, 1)
+            nn.Conv3d(128, final_output_channels, 1, 1, 0)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, final_output_channels, 3, 1, 1)
+            nn.Conv3d(256, final_output_channels, 1, 1, 0)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -257,16 +257,16 @@ class Deep_Supervision_UNet3d(nn.Module):
         )
 
         self.final_layer1 = nn.Sequential(
-            nn.Conv3d(32, final_output_channels, 3, 1, 1)
+            nn.Conv3d(32, final_output_channels, 1, 1, 0) #DeepLab Architecture로 수정
             )
         self.final_layer2 = nn.Sequential(
-            nn.Conv3d(64, final_output_channels, 3, 1, 1)
+            nn.Conv3d(64, final_output_channels, 1, 1, 0)
             )
         self.final_layer3 = nn.Sequential(
-            nn.Conv3d(128, final_output_channels, 3, 1, 1)
+            nn.Conv3d(128, final_output_channels, 1, 1, 0)
             )
         self.final_layer4 = nn.Sequential(
-            nn.Conv3d(256, final_output_channels, 3, 1, 1)
+            nn.Conv3d(256, final_output_channels, 1, 1, 0)
             )
 
         self.upsample4 = nn.Upsample(scale_factor=8, mode = 'trilinear')
@@ -306,7 +306,94 @@ class Deep_Supervision_UNet3d(nn.Module):
         min_patch_axis = np.min(self.patch_size)
         return np.clip(np.log2(min_patch_axis), 0, 6).astype(int)
     
+class Deep_Supervision_UNet3d_Lung(nn.Module):
+    def __init__(self, n_modalities = 4, final_output_channels = 5, patch_size = (128, 128, 128), is_binary = False):
+        super().__init__()
+        self.patch_size = np.array(patch_size)
+        self.networks_depth = self.get_networks_depth()
+        self.encoder_blocks = []
+        self.decoder_blocks = []
+        self.upsample_operations = []
+        self.final_output_channels = final_output_channels
+        self.is_binary = is_binary
+        
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        self.encoder1 = nn.Sequential(
+            ConvDropoutNormNonlin(n_modalities, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(32, 32, conv_kwargs = conv_kwargs1, conv_op = nn.Conv3d)
+        )
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 2}
+        conv_kwargs2 = {'kernel_size' : 3, 'padding' : 1, 'stride' : 1}
+        self.encoder2 = nn.Sequential(
+            ConvDropoutNormNonlin(32, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder3 = nn.Sequential(
+            ConvDropoutNormNonlin(64, 128, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder4 = nn.Sequential(
+            ConvDropoutNormNonlin(128, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(256, 256, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.encoder5 = nn.Sequential(
+            ConvDropoutNormNonlin(256, 320, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(320, 320, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
 
+        conv_kwargs1 = {'kernel_size' : 3, 'padding' : 1,  'stride' : 1}
+        conv_kwargs2 = {'kernel_size' : 3, 'stride' : 1, 'padding':1}
+        transp_kwargs = {'kernel_size' : 2, 'stride' : 1, 'stride' : 2}
+
+
+        self.decoder1_trans = ConvDropoutNormNonlin(320, 256, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder1_conv = nn.Sequential(
+            ConvDropoutNormNonlin(256*2, 256, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(256, 256, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder2_trans = ConvDropoutNormNonlin(256, 128, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder2_conv = nn.Sequential(
+            ConvDropoutNormNonlin(128*2, 128 , conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(128, 128, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder3_trans = ConvDropoutNormNonlin(128, 64, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder3_conv = nn.Sequential(
+            ConvDropoutNormNonlin(64*2, 64, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(64, 64, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.decoder4_trans = ConvDropoutNormNonlin(64, 32, conv_kwargs= transp_kwargs, conv_op = nn.ConvTranspose3d)
+        self.decoder4_conv = nn.Sequential(
+            ConvDropoutNormNonlin(32*2, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs2, conv_op = nn.Conv3d)
+        )
+
+        self.final_block = nn.Sequential(
+            ConvDropoutNormNonlin(32, 32, conv_kwargs= conv_kwargs1, conv_op = nn.Conv3d),
+            ConvDropoutNormNonlin(32, 32, conv_kwargs = conv_kwargs2, conv_op = nn.Conv3d)
+        )
+        self.final_layer = nn.Sequential(
+            nn.Conv3d(32, final_output_channels, 1, 1, 0)
+        )
+        
+    
+    def forward(self, x):
+        x1 = self.encoder1(x)
+        x2 = self.encoder2(x1)
+        x3 = self.encoder3(x2)
+        x4 = self.encoder4(x3)
+        x5 = self.encoder5(x4)
+
+        x4_ = self.decoder1_conv(torch.cat((x4, self.decoder1_trans(x5)), dim = 1))
+        x3_ = self.decoder2_conv(torch.cat((x3, self.decoder2_trans(x4_)), dim = 1))
+        x2_ = self.decoder3_conv(torch.cat((x2, self.decoder3_trans(x3_)), dim = 1))
+        x1_ = self.decoder4_conv(torch.cat((x1, self.decoder4_trans(x2_)), dim = 1))
+        x1_ = self.final_block(x1_)
+        
+        return self.final_layer(x1_)
+                
+    def get_networks_depth(self):
+        min_patch_axis = np.min(self.patch_size)
+        return np.clip(np.log2(min_patch_axis), 0, 6).astype(int)
 
 class Interactive_Model(nn.Module):
     def __init__(self, n_modalities = 1, point_channel = 2, patch_size = (320,256,256), final_output_channels = 1):
