@@ -5,14 +5,14 @@ import numpy as np
 import math
 
 class Batch_Affine_3D(object):
-    def __init__(self, degree = [30, 30, 30], axis = [0,1,2], scale = [0.7, 1.4], use_gpu = True, is_3d_img = True, device = None, cal_time = True):
+    def __init__(self, degree = [30, 30, 30], axis = [0,1,2], scale = [0.7, 1.4], use_gpu = True, device = None, is_random = True):
         self.use_gpu = use_gpu
         self.transform_matrices = []
-        
         self.axis = axis
         self.degree = np.array(degree)
         self.scale = np.array(scale)
         self.device = device
+        self.is_random = is_random
 
     def get_matrices_and_coords(self, images):
         if len(images.shape) == 5:
@@ -29,7 +29,6 @@ class Batch_Affine_3D(object):
 
     
     def __call__(self, images):
-
         if len(images.shape) == 5:
             seg = False
         else:
@@ -74,13 +73,19 @@ class Batch_Affine_3D(object):
         return torch.vstack((grid_x.unsqueeze_(0), grid_y.unsqueeze_(0), grid_z.unsqueeze_(0)))
     
     def set_random_scale_degree(self):
-        degree = np.random.uniform( size = self.degree.shape) * self.degree
-        if np.any(self.scale):
-            scale = np.random.uniform(low = self.scale[0], high = self.scale[1], size = (1,3))
-        
+
+        if self.is_random:
+            degree = np.random.uniform( size = self.degree.shape) * self.degree
+            if np.any(self.scale):
+                scale = np.random.uniform(low = self.scale[0], high = self.scale[1], size = (1,3))
+            else:
+                scale = 1
+            return scale, degree
+    
         else:
+            degree = np.array(self.degree)
             scale = 1
-        return scale, degree
+            return degree, scale
 
     def get_matrices(self):
         transform_matrices = []
